"""Course containers.
"""
import grok
from zope.interface import implementedBy
from zope.component.interfaces import IFactory
from waeup.interfaces import ICourseContainer, ICourse
from waeup.viewlets import MainArea, Index, Manage, FormWrapMixin
from waeup.widgets.interfaces import ITableProvider

class CourseContainer(grok.Container):
    """See interfaces for description.
    """
    grok.implements(ICourseContainer)
    grok.require('waeup.manageUniversity')

    def addCourse(self, course):
        if not ICourse.providedBy(course):
            raise TypeError('CourseContainers contain only ICourse instances')
        self[course.code] = course
        return

    def clear(self):
        keys = self.keys()
        for key in keys:
            del self[key]

class CourseContainerFactory(grok.GlobalUtility):
    """A factory for course containers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.CourseContainer')
    title = u"Create a new course container.",
    description = u"This factory instantiates new course containers."

    def __call__(self, *args, **kw):
        return CourseContainer(*args, **kw)

    def getInterfaces(self):
        return implementedBy(CourseContainer)

class DisplayForm(grok.DisplayForm):
    grok.context(ICourseContainer)
    form_fields = grok.AutoFields(ICourseContainer)

    
class Content(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddDepartmentForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(ICourseContainer)
    grok.view(Index)
    grok.require('waeup.manageUniversity')

    formview_name = 'displayform' # The name of the formview we want
                                  # to be rendered in this viewlet.

class ManageForm(grok.EditForm):
    """Manage the basic properties of a `Coursecontainer` instance.
    """

    form_fields = grok.AutoFields(ICourseContainer)

    @property
    def label(self):
        # Set department name as form label
        #return self.form_fields['title'].field.get(self.context)
        return ''
    
    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageCourseContainer(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(ICourseContainer)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    formview_name = 'manageform' # The name of the formview we want to
                                 # be rendered in this viewlet.

class CourseList(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(ICourseContainer)
    grok.view(Index)
    grok.order(3)

    def update(self):
        self.table = ITableProvider(self.context).getTable(view=self.view)
        self.table.need()
